from danila.danila_rama_text_recognize_base import Danila_rama_text_recognize_base


class Danila_rama_text_recognize:
    def __init__(self, yolov5_dir, danila_rama_text_recognize_params):
        self.danila_rama_text_recognize_params = danila_rama_text_recognize_params
        self.danila_rama_text_recognize = Danila_rama_text_recognize_base(yolov5_dir,
                                                                    self.danila_rama_text_recognize_params.rama_text_recognize_version
                                                                    )

    def rama_text_recognize(self, rama_prod, img_cut, img_text_areas):
        return self.danila_rama_text_recognize.rama_text_recognize(rama_prod, img_cut, img_text_areas)


from enum import Enum


class Rama_Prod(Enum):
    altai = 0
    balakovo = 1
    begickaya = 2
    promlit = 3
    ruzhimmash = 4
    tihvin = 5
    uralvagon = 6
    no_rama = 7

class Rama_Prod_Conf:

    def __init__(self, rama_prod, conf):
        self.rama_prod = rama_prod
        self.conf = conf


from data.neuro.Balka_prod_classify_class import Balka_prod_classify_class
from data.neuro.models import BALKA_CLASSIFY_MODEL_ADDRESS


class Danila_balka_classify_prod:
    def __init__(self, yolov5_dir, balka_classify_model):
        yolo_path = yolov5_dir
        if balka_classify_model == 1:
            balka_prod_classify_model_path = BALKA_CLASSIFY_MODEL_ADDRESS
        print('reading and loading - BALKA_PROD_CLASSIFY_MODEL')
        self.balka_prod_classify_model = Balka_prod_classify_class(balka_prod_classify_model_path, yolo_path)

    def balka_classify(self, img):
        balka_prod_conf = self.balka_prod_classify_model.classify(img)
        return balka_prod_conf


from danila.danila_rama_classify_prod import Danila_rama_classify_prod
from data.neuro.prods import RAMA_PRODS
from data.result.Rama_prod import Rama_Prod
from data.result.Text_cut_recognize_result import Text_cut_recognize_result
from data.result.Text_recognize_result import Text_recognize_result


class Danila_rama_classify:
    def __init__(self, yolov5_dir, danila_rama_classify_params):
        self.danila_rama_classify_params = danila_rama_classify_params
        if self.danila_rama_classify_params.rama_classify_version == 1:
            self.danila_rama_classify = Danila_rama_classify_prod(yolov5_dir, self.danila_rama_classify_params.rama_classify_model)

    def rama_classify(self, img):
        return self.danila_rama_classify.rama_classify(img)




from danila.danila_balka_classify_prod import Danila_balka_classify_prod


class Danila_balka_classify:
    def __init__(self, yolov5_dir, danila_balka_classify_params):
        self.danila_balka_classify_params = danila_balka_classify_params
        if self.danila_balka_classify_params.balka_classify_version == 1:
            self.danila_balka_classify = Danila_balka_classify_prod(yolov5_dir, self.danila_balka_classify_params.balka_classify_model)


    def balka_classify(self, img):
        return self.danila_balka_classify.balka_classify(img)


from danila.danila_rama_text_detect_base import Danila_rama_text_detect_base


class Danila_rama_text_detect:
    def __init__(self, yolov5_dir, danila_rama_text_detect_params):
        self.danila_rama_text_detect_params = danila_rama_text_detect_params
        self.danila_rama_text_detect = Danila_rama_text_detect_base(yolov5_dir,
                                                                    self.danila_rama_text_detect_params.rama_detect_version,
                                                                    self.danila_rama_text_detect_params.rama_text_detect_version)

    def rama_text_detect(self, img, rama_prod):
        return self.danila_rama_text_detect.rama_text_detect(img, rama_prod)

    def rama_rext_detect_cuts(self, img, rama_prod):
        return self.danila_rama_text_detect.rama_text_detect_cuts(img, rama_prod)


import json
import os
import zipfile

import cv2
import requests
from urllib.parse import urlencode
import torch
import hashlib
from datetime import datetime
from data.neuro.letters_in_image import Letters_In_Image
from data.neuro.objs_in_image import Objs_In_Image
from data.result.Class_text import Class_text
from data.result.Label_area import Label_area
from data.result.Rama_prod import Rama_Prod, Rama_Prod_Conf
from data.result.balka_prod import Balka_Prod_Conf, Balka_Prod


class Balka_prod_classify_class:

    def __init__(self, model_path, yolo_path):
        """reads yolov5 taught model from yandex-disk and includes it in class example"""
        base_url = 'https://cloud-api.yandex.net/v1/disk/public/resources/download?'
        public_key = model_path  # Сюда вписываете вашу ссылку
        # Получаем загрузочную ссылку
        final_url = base_url + urlencode(dict(public_key=public_key))
        response = requests.get(final_url)
        download_url = response.json()['href']
        # Загружаем файл и сохраняем его
        download_response = requests.get(download_url)
        zip_path = 'balka_classify.zip'
        # print(download_response.content)
        with open(zip_path, 'wb') as f:
            f.write(download_response.content)

        with zipfile.ZipFile(zip_path, 'r') as zip_ref:
            zip_ref.extractall()
        # weights_file_path = model_name + '_weights.pt'
        self.model = torch.hub.load(yolo_path, 'custom', 'balka_classify.pt', source='local')

        # for every text_class try to recognize text from all areas of text_class, length is depends on class and prod, returns string


    def work_image(self, img, size):
        hash_object = hashlib.md5(bytes(str(datetime.now()), 'utf-8'))
        hash_str = hash_object.hexdigest()
        img_name = 'img' + hash_str + '.jpg'
        cv2.imwrite(img_name, img)
        results = self.model([img_name], size = size)
        json_res = results.pandas().xyxy[0].to_json(orient="records")
        res2 = json.loads(json_res)
        if len(res2) == 0:
            result = Balka_Prod_Conf(Balka_Prod(5), 0.0)
        else:
            res_index = res2[0]['class']
            res_conf = res2[0]['confidence']
            result = Balka_Prod_Conf(Balka_Prod(res_index), res_conf)
        os.remove(img_name)
        return result

    def classify(self, img):
        self.model.max_det = 1
        balka_prod_conf = self.work_image(img, 640)
        if balka_prod_conf.balka_prod == Balka_Prod.no_balka:
            sizes = [224, 256, 288, 320, 352, 384, 416, 448, 480, 512, 544, 576, 608, 640, 672, 704, 736, 768, 800]
            flag = True
            for size in sizes:
                if flag:
                    balka_prod_conf1 = self.work_image(img, size)
                    flag = (balka_prod_conf1.balka_prod == Balka_Prod.no_balka)
                    if not (flag):
                        balka_prod_conf = balka_prod_conf1
        return balka_prod_conf


from danila.danila_vagon_text_detect_base import Danila_vagon_text_detect_base
from danila.danila_vagon_text_recognize_base import Danila_vagon_text_recognize_base


class Danila_vagon_text_recognize:
    def __init__(self, yolov5_dir, danila_vagon_text_recognize_params):
        self.danila_vagon_text_recognize = Danila_vagon_text_recognize_base(yolov5_dir, danila_vagon_text_recognize_params)

    def vagon_number_recognize(self, img, vagon_number_rects):
        return self.danila_vagon_text_recognize.vagon_number_recognize(img, vagon_number_rects)


class Danila_rama_classify_params:
    def __init__(self, rama_classify_version, rama_classify_model):
        self.rama_classify_version = rama_classify_version
        self.rama_classify_model = rama_classify_model


from danila.Danila_balka_classify import Danila_balka_classify
from danila.danila_balka_text_detect import Danila_balka_text_detect
from data.neuro.prods import RAMA_PRODS, BALKA_PRODS
from data.result.Text_cut_recognize_result import Text_cut_recognize_result
from data.result.Text_recognize_result import Text_recognize_result
from data.result.balka_prod import Balka_Prod


class Danila_balka:
    def __init__(self, yolov5_dir, danila_balka_params):
        self.danila_balka_params = danila_balka_params
        self.danila_balka_classify = Danila_balka_classify(yolov5_dir, self.danila_balka_params.danila_balka_classify_params)
        self.danila_balka_text_detect = Danila_balka_text_detect(yolov5_dir,
                                                               self.danila_balka_params.danila_balka_text_detect_params)


    def balka_classify(self,img, detail):
        balka_prod_conf = self.danila_balka_classify.balka_classify(img)
        if detail is None:
            detail_prod = Text_cut_recognize_result('balka', 1)
            det = Text_recognize_result(detail_prod)
        else:
            det = detail
        if balka_prod_conf.balka_prod != Balka_Prod.no_balka:
            text_prod = BALKA_PRODS[balka_prod_conf.balka_prod]
            det.prod = Text_cut_recognize_result(text_prod, balka_prod_conf.conf)
        return det

    def balka_text_detect(self, img):
        balka_prod_conf = self.danila_balka_classify.balka_classify(img)
        result_img = img
        if balka_prod_conf.balka_prod != Balka_Prod.no_balka:
            result_img = self.danila_balka_text_detect.balka_text_detect(img, balka_prod_conf.balka_prod)
        return result_img


import hashlib
import os
from datetime import datetime

import cv2

from data.neuro.Detail_classify_class import Detail_classify_class
from data.neuro.Letters_recognize import Letters_recognize
from data.neuro.Rama_detect_class import Rama_detect_class
from data.neuro.Rama_prod_classify_class import Rama_prod_classify_class
from data.neuro.Rama_text_detect_class import Rama_text_detect_class
from data.neuro.Vagon_number_detect_class import Vagon_number_detect_class
from data.neuro.Vagon_number_recognize_class import Vagon_number_recognize_class
from data.neuro.Vagon_number_recognize_yolo import Vagon_number_recognize_yolo
from data.neuro.models import *
from data.result.Class_im import Class_detail
from data.result.Class_text import Class_text
from data.result.Rama_prod import Rama_Prod
from data.result.Rect import Rect
from data.neuro.text_recognize_yolo import Text_Recognize_yolo

"""main module for user"""


class Text_recognize_result:
    def __init__(self, detail, number = None, prod = None, year = None):
        self.detail = detail
        self.number = number
        self.prod = prod
        self.year = year

class Text_cut_recognize_result:
    def __init__(self, text, conf):
        self.text = text
        self.conf = conf

class Danila_v10:
    """main class for user"""
    def __init__(self, yolov5_dir):
        detail_classify_model_path = DETAIL_CLASSIFY_MODEL_ADDRESS
        print('reading and loading - DETAIL_CLASSIFY_MODEL')
        self.detail_classify_model = Detail_classify_class(detail_classify_model_path)
        yolo_path = yolov5_dir
        rama_prod_classify_model_path = RAMA_PROD_CLASSIFY_MODEL_ADDRESS
        print('reading and loading - RAMA_PROD_CLASSIFY_MODEL')
        self.rama_prod_classify_model = Rama_prod_classify_class(rama_prod_classify_model_path, yolo_path)
        rama_no_spring_detect_model_path = RAMA_BEGICKAYA_DETECT_MODEL_ADDRESS
        print('reading and loading - RAMA_BEGICKAYA_DETECT_MODEL')
        self.rama_no_spring_detect_model = Rama_detect_class(rama_no_spring_detect_model_path,
                                                              yolo_path)
        rama_spring_detect_model_path = RAMA_RUZHIMMASH_DETECT_MODEL_ADDRESS
        print('reading and loading - RAMA_RUZHIMMASH_DETECT_MODEL')
        self.rama_spring_detect_model = Rama_detect_class(rama_spring_detect_model_path,
                                                             yolo_path)
        rama_spring_ruzhimmash_text_detect_model_path = RAMA_RUZHIMMASH_TEXT_DETECT_MODEL_ADDRESS
        print('reading and loading - RAMA_RUZHIMMASH_TEXT_DETECT_MODEL')
        self.rama_spring_ruzhimmash_text_detect_model = Rama_text_detect_class(rama_spring_ruzhimmash_text_detect_model_path,
                                                                           yolo_path)
        rama_no_spring_bejickaya_text_detect_model_path = RAMA_BEGICKAYA_TEXT_DETECT_MODEL_ADDRESS
        print('reading and loading - RAMA_BEJICKAYA_TEXT_DETECT_MODEL')
        self.rama_no_spring_bejickaya_text_detect_model = Rama_text_detect_class(rama_no_spring_bejickaya_text_detect_model_path,
                                                                             yolo_path)
        text_recognize_yolo_ruzhimmash_model_path = TEXT_RECOGNIZE_RUZHIMMASH_MODEL_ADDRESS
        print('reading and loading - TEXT_RECOGNIZE_RUZHIMMASH_MODEL')
        self.text_recognize_ruzhimmash_model = Text_Recognize_yolo(text_recognize_yolo_ruzhimmash_model_path, yolo_path)
        text_recognize_yolo_begickaya_model_path = TEXT_RECOGNIZE_BEGICKAYA_MODEL_ADDRESS
        print('reading and loading - TEXT_RECOGNIZE_BEGICKAYA_MODEL')
        self.text_recognize_begickaya_model = Text_Recognize_yolo(text_recognize_yolo_begickaya_model_path, yolo_path)
        vagon_number_detect_model_path = VAGON_NUMBER_DETECT_MODEL_ADDRESS_2
        print('reading and loading - VAGON_NUMBER_DETECT_MODEL')
        self.vagon_number_detect_model = Vagon_number_detect_class(vagon_number_detect_model_path, yolo_path)
        print('loading - VAGON_NUMBER_RECOGNIZE_MODEL')
        vagon_number_recognize_model_path = VAGON_NUMBER_RECOGNIZE_MODEL_ADDRESS_4
        self.vagon_number_recognize_model = Vagon_number_recognize_yolo(vagon_number_recognize_model_path, yolo_path)

    def detail_classify(self, img, size = 256):
        class_detail_conf = self.detail_classify_model.classify(img)
        detail = Text_recognize_result(Text_cut_recognize_result(class_detail_conf.class_detail.name, class_detail_conf.conf))
        if class_detail_conf.class_detail == Class_detail.rama:
            detail = self.rama_classify(img, size, detail)
        return detail

    def detail_text_detect(self, img, size = 256):
        detail = self.detail_classify(img, size)
        if detail.detail.text == 'rama':
            img_res = self.rama_text_detect_cut(img, size,detail)
        else:
            img_res = self.vagon_number_detect(img, size, detail)
        return img_res

    def detail_text_recognize(self, img, size = 256):
        detail = self.detail_classify(img,size)
        if detail.detail.text == 'rama':
            res = self.rama_text_recognize(img, size,detail)
        else:
            res = self.vagon_number_recognize(img, size, detail)
        return res


    # returns Rama_Prod_Conf - class of rama and confidence using CNN network
    # img - openCV frame
    def rama_classify(self, img, size = 256, detail = None):
        """rama_classify(Img : openCv frame): String - returns class of rama using CNN network"""
        """rama_classify uses Rama_classify_class method - classify(Img)"""
        # img = cv2.imread(img_path)

        rama_prod_conf = self.rama_prod_classify_model.classify(img, size)
        rama_prod = rama_prod_conf.rama_prod
        if rama_prod == Rama_Prod.no_rama:
            sizes = [256,384,512,640]
            flag = True
            for s in sizes:
                if flag:
                    rama_prod_conf1 = self.rama_prod_classify_model.classify(img, s)
                    rama_prod1 = rama_prod_conf1.rama_prod
                    if (rama_prod1 != Rama_Prod.no_rama):
                        rama_prod_conf = rama_prod_conf1
                        flag = False
        if detail is None:
            detail_prod = Text_cut_recognize_result('rama', 1)
            det = Text_recognize_result(detail_prod)
        else:
            det = detail
        if rama_prod_conf.rama_prod != Rama_Prod.no_rama:
            if rama_prod_conf.rama_prod == Rama_Prod.ruzhimmash:
                text_prod = '1275'
            else:
                text_prod = '12'
            det.prod = Text_cut_recognize_result(text_prod, rama_prod_conf.conf)
        return det


    # returns openCV cut rama with drawn text areas
    def rama_text_detect_cut(self, img, size = 256, detail = None):
        """returns openCV cut rama with drawn text areas"""
        det = self.rama_classify(img, size, detail)
        if det.prod is None:
            return img
        elif det.prod.text == '1275':
            rama_prod = Rama_Prod.ruzhimmash
        else:
            rama_prod = Rama_Prod.bejickaya
        hash_object = hashlib.md5(bytes(str(datetime.now()), 'utf-8'))
        hash_str = hash_object.hexdigest()
        initial_image_path = 'initial_image' + hash_str + '.jpg'
        cv2.imwrite(initial_image_path, img)
        rect = Rect()
        if (rama_prod == Rama_Prod.ruzhimmash):
            rect = self.rama_spring_detect_model.rama_detect(initial_image_path)
            if rect is None:
                os.remove(initial_image_path)
                return img
            img_cut = img[rect.ymin:rect.ymax, rect.xmin:rect.xmax]
            hash_object = hashlib.md5(bytes(str(datetime.now()), 'utf-8'))
            hash_str = hash_object.hexdigest()
            img_cut_path = 'cut_img' + hash_str + '.jpg'
            cv2.imwrite(img_cut_path, img_cut)
            image_text_areas = self.rama_spring_ruzhimmash_text_detect_model.text_detect(img_cut_path)
            image_drawn_text_areas = self.rama_spring_ruzhimmash_text_detect_model.draw_text_areas_in_opencv(image_text_areas, img_cut)
            os.remove(initial_image_path)
            os.remove(img_cut_path)
            return image_drawn_text_areas
        else:
            rect = self.rama_no_spring_detect_model.rama_detect(initial_image_path)
            if rect is None:
                os.remove(initial_image_path)
                return img
            img_cut = img[rect.ymin:rect.ymax, rect.xmin:rect.xmax]
            hash_object = hashlib.md5(bytes(str(datetime.now()), 'utf-8'))
            hash_str = hash_object.hexdigest()
            img_cut_path = 'cut_img' + hash_str + '.jpg'
            cv2.imwrite(img_cut_path, img_cut)
            image_text_areas = self.rama_no_spring_bejickaya_text_detect_model.text_detect(img_cut_path)
            image_drawn_text_areas = self.rama_no_spring_bejickaya_text_detect_model.draw_text_areas_in_opencv(image_text_areas, img_cut)
            os.remove(initial_image_path)
            os.remove(img_cut_path)
            return image_drawn_text_areas

    # returns dict {'number', 'prod', 'year'} for openCV rama img or 'no_rama'

    def rama_text_recognize(self, img, size = 256, detail = None):
        """returns dict {'number', 'prod', 'year'} for openCV rama img or 'no_rama'"""
        det = self.rama_classify(img, size, detail)
        if det.prod is None:
            return det
        elif det.prod.text == '1275':
            rama_prod = Rama_Prod.ruzhimmash
        else:
            rama_prod = Rama_Prod.bejickaya
        hash_object = hashlib.md5(bytes(str(datetime.now()), 'utf-8'))
        hash_str = hash_object.hexdigest()
        initial_image_path = 'initial_image' + hash_str + '.jpg'
        cv2.imwrite(initial_image_path, img)
        rect = Rect()
        if (rama_prod == Rama_Prod.ruzhimmash):
            rect = self.rama_spring_detect_model.rama_detect(initial_image_path)
            if rect is None:
                os.remove(initial_image_path)
                return det
            img_cut = img[rect.ymin:rect.ymax, rect.xmin:rect.xmax]
            hash_object = hashlib.md5(bytes(str(datetime.now()), 'utf-8'))
            hash_str = hash_object.hexdigest()
            img_cut_path = 'cut_img' + hash_str + '.jpg'
            cv2.imwrite(img_cut_path, img_cut)
            image_text_areas = self.rama_spring_ruzhimmash_text_detect_model.text_detect(img_cut_path)

            label_area = self.text_recognize_ruzhimmash_model.work_image_cut(image_text_areas, img_cut, 5, 64,128,
                                                                  4, 64,128, 2, 64,64)
            res_labels = {}
            (number_text, number_conf) = label_area.labels[Class_text.number]
            if len(number_text) == 5:
                res_labels['number'] = (number_text, number_conf)
            else:
                number_image_text_areas = image_text_areas.areas[Class_text.number]
                image_text_areas_min = []
                image_text_areas_max = []
                for number_image_text_area in number_image_text_areas:
                    w = number_image_text_area.xmax - number_image_text_area.xmin
                    if (number_image_text_area.xmin - (w // 2)) < 0:
                        new_xmin = 0
                    else:
                        new_xmin = number_image_text_area.xmin - (w // 2)
                    rect_min = Rect(xmin=new_xmin, xmax=number_image_text_area.xmax, ymin=number_image_text_area.ymin,
                                    ymax=number_image_text_area.ymax)
                    image_text_areas_min.append(rect_min)
                    new_xmax = number_image_text_area.xmax + w // 2
                    rect_max = Rect(xmin=number_image_text_area.xmin, xmax=new_xmax, ymin=number_image_text_area.ymin,
                                    ymax=number_image_text_area.ymax)
                    image_text_areas_max.append(rect_max)
                image_text_areas.areas[Class_text.number] = image_text_areas_min
                label_area_min = self.text_recognize_begickaya_model.work_image_cut(image_text_areas, img_cut, 5, 64,128,
                                                                  4, 64,128, 2, 64,64)
                (number_text_min, number_conf_min) = label_area_min.labels[Class_text.number]
                if len(number_text_min) == 5:
                    res_labels['number'] = (number_text_min, number_conf_min)
                else:
                    image_text_areas.areas[Class_text.number] = image_text_areas_max
                    label_area_max = self.text_recognize_begickaya_model.work_image_cut(image_text_areas, img_cut, 5, 64,128,
                                                                  4, 64,128, 2, 64,64)
                    (number_text_max, number_conf_max) = label_area_max.labels[Class_text.number]
                    if len(number_text_max) == 5:
                        res_labels['number'] = (number_text_max, number_conf_max)
                    else:
                        res_labels['number'] = (number_text, number_conf)
            (year_text, year_conf) = label_area.labels[Class_text.year]
            if (len(year_text) == 2) and (int(year_text) < 25):
                res_labels['year'] = (year_text, year_conf)
            else:
                res_labels['year'] = ('23', 0.25)
            os.remove(initial_image_path)
            os.remove(img_cut_path)
            res_labels_number_text, res_labels_number_conf = res_labels['number']
            det.number = Text_cut_recognize_result(res_labels_number_text, res_labels_number_conf)
            res_labels_year_text, res_labels_year_conf = res_labels['year']
            det.year = Text_cut_recognize_result(res_labels_year_text, res_labels_year_conf)
            return det
        else:
            rect = self.rama_no_spring_detect_model.rama_detect(initial_image_path)
            if rect is None:
                os.remove(initial_image_path)
                return det
            img_cut = img[rect.ymin:rect.ymax, rect.xmin:rect.xmax]
            hash_object = hashlib.md5(bytes(str(datetime.now()), 'utf-8'))
            hash_str = hash_object.hexdigest()
            img_cut_path = 'cut_img' + hash_str + '.jpg'
            cv2.imwrite(img_cut_path, img_cut)
            image_text_areas = self.rama_no_spring_bejickaya_text_detect_model.text_detect(img_cut_path)
            label_area = self.text_recognize_begickaya_model.work_image_cut(image_text_areas, img_cut, 6, 64, 192,
                                                                  2, 64, 64, 2, 64,64)
            res_labels = {}
            (number_text, number_conf) = label_area.labels[Class_text.number]
            if len(number_text) == 6:
                res_labels['number'] = (number_text, number_conf)
            else:
                number_image_text_areas = image_text_areas.areas[Class_text.number]
                image_text_areas_min = []
                image_text_areas_max = []
                for number_image_text_area in number_image_text_areas:
                    w = number_image_text_area.xmax - number_image_text_area.xmin
                    if (number_image_text_area.xmin - (w//2)) < 0:
                        new_xmin = 0
                    else:
                        new_xmin = number_image_text_area.xmin - (w//2)
                    rect_min = Rect(xmin=new_xmin, xmax=number_image_text_area.xmax, ymin=number_image_text_area.ymin, ymax=number_image_text_area.ymax)
                    image_text_areas_min.append(rect_min)
                    new_xmax = number_image_text_area.xmax + w//2
                    rect_max = Rect(xmin=number_image_text_area.xmin, xmax=new_xmax, ymin=number_image_text_area.ymin, ymax=number_image_text_area.ymax)
                    image_text_areas_max.append(rect_max)
                image_text_areas.areas[Class_text.number] = image_text_areas_min
                label_area_min = self.text_recognize_begickaya_model.work_image_cut(image_text_areas, img_cut, 6, 64, 192,
                                                                  2, 64, 64, 2, 64,64)
                (number_text_min, number_conf_min) = label_area_min.labels[Class_text.number]
                if len(number_text_min) == 6:
                    res_labels['number'] = (number_text_min, number_conf_min)
                else:
                    image_text_areas.areas[Class_text.number] = image_text_areas_max
                    label_area_max = self.text_recognize_begickaya_model.work_image_cut(image_text_areas, img_cut, 6, 64, 192,
                                                                  2, 64, 64, 2, 64,64)
                    (number_text_max, number_conf_max) = label_area_max.labels[Class_text.number]
                    if len(number_text_max) == 6:
                        res_labels['number'] = (number_text_max, number_conf_max)
                    else:
                        res_labels['number'] = (number_text, number_conf)
            (year_text, year_conf) = label_area.labels[Class_text.year]
            if (len(year_text) == 2) and (int(year_text) < 25):
                res_labels['year'] = (year_text, year_conf)
            else:
                res_labels['year'] = ('23', 0.25)
            os.remove(initial_image_path)
            os.remove(img_cut_path)
            res_labels_number_text, res_labels_number_conf = res_labels['number']
            det.number = Text_cut_recognize_result(res_labels_number_text, res_labels_number_conf)
            res_labels_year_text, res_labels_year_conf = res_labels['year']
            det.year = Text_cut_recognize_result(res_labels_year_text, res_labels_year_conf)
            return det


    def vagon_number_detect(self, img, size = 256, detail = None):
        hash_object = hashlib.md5(bytes(str(datetime.now()), 'utf-8'))
        hash_str = hash_object.hexdigest()
        initial_image_path = 'initial_image' + hash_str + '.jpg'
        cv2.imwrite(initial_image_path, img)
        number_rects = self.vagon_number_detect_model.vagon_rama_detect(initial_image_path, size)
        if len(number_rects) == 0:
            os.remove(initial_image_path)
            return img
        img_with_number = img.copy()
        for number_rect in number_rects:
            cv2.rectangle(img_with_number, (number_rect.xmin, number_rect.ymin), (number_rect.xmax, number_rect.ymax), (0, 0, 255), 2)
            cv2.putText(img_with_number, 'number', (number_rect.xmin, number_rect.ymin), cv2.FONT_HERSHEY_SIMPLEX, 0.5, (0, 0, 255), 2)
        return img_with_number

    def vagon_number_recognize(self, img, size = 256, detail = None, size_number_h = 128, size_number_w = 320):
        det = detail
        hash_object = hashlib.md5(bytes(str(datetime.now()), 'utf-8'))
        hash_str = hash_object.hexdigest()
        initial_image_path = 'initial_image' + hash_str + '.jpg'
        cv2.imwrite(initial_image_path, img)
        vagon_number_rects = self.vagon_number_detect_model.vagon_rama_detect(initial_image_path, size)
        if det is None:
            det = Text_recognize_result(Text_cut_recognize_result('vagon',1.0))
        if len(vagon_number_rects)==0:
            os.remove(initial_image_path)
            return detail
        detail_number_text, detail_number_conf = self.vagon_number_recognize_model.work_image(img,vagon_number_rects, size_number_h, size_number_w)
        det.number = Text_cut_recognize_result(detail_number_text, round(detail_number_conf,2))
        os.remove(initial_image_path)
        return det



class Danila_rama_text_detect_params:
    def __init__(self, rama_detect_version, rama_text_detect_version):
        self.rama_detect_version = rama_detect_version
        self.rama_text_detect_version = rama_text_detect_version


import hashlib
import os
from datetime import datetime

import cv2

from data.neuro.Balka_detect_class import Balka_detect_class
from data.neuro.Rama_detect_class import Rama_detect_class
from data.neuro.Rama_text_detect_class import Rama_text_detect_class
from data.neuro.balka_text_detect_class import Balka_text_detect_class
from data.neuro.models import *
from data.result import Rama_prod, balka_prod


class Danila_balka_text_detect_base:
    def __init__(self, yolov5_dir, balka_detect_version, balka_text_detect_version):
        if (balka_detect_version == 1) & (balka_text_detect_version == 1):

            self.balka_detect_objects = {
                 balka_prod.Balka_Prod.altai: None,
                 balka_prod.Balka_Prod.begickaya: None,
                 balka_prod.Balka_Prod.promlit: None,
                 balka_prod.Balka_Prod.ruzhimmash: None,
                 balka_prod.Balka_Prod.tihvin: None
            }

            print('reading and loading - BALKA_ALTAI_DETECT_MODEL')
            self.balka_detect_objects[balka_prod.Balka_Prod.altai] = Balka_detect_class(BALKA_ALTAI_DETECT_MODEL_ADDRESS, yolov5_dir)

            print('reading and loading - BALKA_BEGICKAYA_DETECT_MODEL')
            self.balka_detect_objects[balka_prod.Balka_Prod.begickaya] = Balka_detect_class(BALKA_BEGICKAYA_DETECT_MODEL_ADDRESS, yolov5_dir)

            print('reading and loading - BALKA_PROMLIT_DETECT_MODEL')
            self.balka_detect_objects[balka_prod.Balka_Prod.promlit] = Balka_detect_class(BALKA_PROMLIT_DETECT_MODEL_ADDRESS, yolov5_dir)

            print('reading and loading - BALKA_RUZHIMMASH_DETECT_MODEL')
            self.balka_detect_objects[balka_prod.Balka_Prod.ruzhimmash] = Balka_detect_class(BALKA_RUZHIMMASH_DETECT_MODEL_ADDRESS, yolov5_dir)

            print('reading and loading - BALKA_TIHVIN_DETECT_MODEL')
            self.balka_detect_objects[balka_prod.Balka_Prod.tihvin] = Balka_detect_class(BALKA_TIHVIN_DETECT_MODEL_ADDRESS,
                                                                                      yolov5_dir)


            self.balka_text_detect_objects = {
                balka_prod.Balka_Prod.altai: None,
                balka_prod.Balka_Prod.begickaya: None,
                balka_prod.Balka_Prod.promlit: None,
                balka_prod.Balka_Prod.ruzhimmash: None,
                balka_prod.Balka_Prod.tihvin: None
            }

            print('reading and loading - BALKA_ALTAI_TEXT_DETECT_MODEL')
            self.balka_text_detect_objects[balka_prod.Balka_Prod.altai] = Balka_text_detect_class(
                BALKA_ALTAI_TEXT_DETECT_MODEL_ADDRESS, yolov5_dir, balka_prod.Balka_Prod.altai)

            print('reading and loading - BALKA_BEGICKAYA_TEXT_DETECT_MODEL')
            self.balka_text_detect_objects[balka_prod.Balka_Prod.begickaya] = Balka_text_detect_class(
                BALKA_BEGICKAYA_TEXT_DETECT_MODEL_ADDRESS, yolov5_dir, balka_prod.Balka_Prod.begickaya)

            print('reading and loading - BALKA_PROMLIT_TEXT_DETECT_MODEL')
            self.balka_text_detect_objects[balka_prod.Balka_Prod.promlit] = Balka_text_detect_class(
                BALKA_PROMLIT_TEXT_DETECT_MODEL_ADDRESS, yolov5_dir, balka_prod.Balka_Prod.promlit)

            print('reading and loading - BALKA_RUZHIMMASH_TEXT_DETECT_MODEL')
            self.balka_text_detect_objects[balka_prod.Balka_Prod.ruzhimmash] = Balka_text_detect_class(
                BALKA_RUZHIMMASH_TEXT_DETECT_MODEL_ADDRESS, yolov5_dir, balka_prod.Balka_Prod.ruzhimmash)

            print('reading and loading - BALKA_TIHVIN_TEXT_DETECT_MODEL')
            self.balka_text_detect_objects[balka_prod.Balka_Prod.tihvin] = Balka_text_detect_class(
                BALKA_TIHVIN_TEXT_DETECT_MODEL_ADDRESS,
                yolov5_dir, balka_prod.Balka_Prod.tihvin)

    def balka_text_detect(self, img, balka_prod):
        hash_object = hashlib.md5(bytes(str(datetime.now()), 'utf-8'))
        hash_str = hash_object.hexdigest()
        initial_image_path = 'initial_balka' + hash_str + '.jpg'
        cv2.imwrite(initial_image_path, img)
        rects = self.balka_detect_objects[balka_prod].balka_detect(initial_image_path)
        if rects == []:
            os.remove(initial_image_path)
            return img
        imgs = []
        for rect in rects:
            imgs.append(img[rect.ymin:rect.ymax, rect.xmin:rect.xmax])
        image_text_areas_half_balkas = self.balka_text_detect_objects[balka_prod].balka_text_detect(imgs)
        images_drawn_text_areas = self.balka_text_detect_objects[balka_prod].draw_text_areas_in_opencv(
            image_text_areas_half_balkas, imgs)
        if len(images_drawn_text_areas) == 1:
            new_img = images_drawn_text_areas[0]
        else:
            h0, w0 = images_drawn_text_areas[0].shape[:2]
            h1, w1 = images_drawn_text_areas[1].shape[:2]
            dsize = (max(h0,h1), w0 + w1)
            new_img = cv2.resize(images_drawn_text_areas[0],dsize)
            left_image = cv2.resize(images_drawn_text_areas[0], (max(h0,h1), w0))
            for i in range(max(h0,h1)):
                for j in range(w0):
                    new_img[i,j] = left_image[i,j]
                for k in range(w0,w0+w1):
                    new_img[i,k] = images_drawn_text_areas[1][i,k - w0]
        os.remove(initial_image_path)
        return new_img

    def balka_text_detect_cuts(self, img, balka_prod):
        hash_object = hashlib.md5(bytes(str(datetime.now()), 'utf-8'))
        hash_str = hash_object.hexdigest()
        initial_image_path = 'initial_balka' + hash_str + '.jpg'
        cv2.imwrite(initial_image_path, img)
        rects = self.balka_detect_objects[balka_prod].balka_detect(initial_image_path)
        if rects == []:
            os.remove(initial_image_path)
            return [], []
        imgs = []
        for rect in rects:
            imgs.append(img[rect.ymin:rect.ymax, rect.xmin:rect.xmax])
        image_text_areas_half_balkas = self.balka_text_detect_objects[balka_prod].balka_text_detect(imgs)
        os.remove(initial_image_path)
        return imgs, image_text_areas_half_balkas


from data.neuro.models import TEXT_RECOGNIZE_RUZHIMMASH_MODEL_ADDRESS
from data.neuro.text_recognize_yolo import Text_Recognize_yolo
from data.result import Rama_prod
from data.result.Class_text import Class_text
from data.result.Rect import Rect


class Prod_coefficients:
    def __init__(self, number_coefficients, prod_coefficients, year_coefficients):
        self.number_coefficients = number_coefficients
        self.prod_coefficients = prod_coefficients
        self.year_coefficients = year_coefficients

class Text_coefficients:
    def __init__(self, length, height, width):
        self.length = length
        self.height = height
        self.width = width

class Danila_rama_text_recognize_base:
    def __init__(self, yolov5_dir, rama_text_recognize_version):

        if rama_text_recognize_version == 1:
            self.prod_coefficients = {
                Rama_prod.Rama_Prod.altai: Prod_coefficients(Text_coefficients(5, 64, 192),
                                                             Text_coefficients(4, 64, 128),
                                                             Text_coefficients(2, 64, 96)),
                Rama_prod.Rama_Prod.balakovo: Prod_coefficients(Text_coefficients(5, 64, 192),
                                                                Text_coefficients(4, 64, 192),
                                                                Text_coefficients(2, 64, 96)),
                Rama_prod.Rama_Prod.begickaya: Prod_coefficients(Text_coefficients(6, 96, 224),
                                                                 Text_coefficients(2, 64, 128),
                                                                 Text_coefficients(2, 64, 96)),
                Rama_prod.Rama_Prod.promlit: Prod_coefficients(Text_coefficients(5, 64, 160),
                                                               Text_coefficients(2, 64, 96),
                                                               Text_coefficients(2, 64, 96)),
                Rama_prod.Rama_Prod.ruzhimmash: Prod_coefficients(Text_coefficients(5, 64, 160),
                                                                  Text_coefficients(4, 64, 192),
                                                                  Text_coefficients(2, 64, 96)),
                Rama_prod.Rama_Prod.tihvin: Prod_coefficients(Text_coefficients(5, 64, 160),
                                                              Text_coefficients(4, 64, 224),
                                                              Text_coefficients(2, 64, 96)),
                Rama_prod.Rama_Prod.uralvagon: Prod_coefficients(Text_coefficients(5, 64, 192),
                                                                 Text_coefficients(1, 64, 64),
                                                                 Text_coefficients(2, 64, 96)),
            }
            print('reading and loading - RAMA_TEXT_RECOGNIZE_MODEL')
            self.text_recognize_model = Text_Recognize_yolo(TEXT_RECOGNIZE_RUZHIMMASH_MODEL_ADDRESS, yolov5_dir)

    def rama_text_recognize(self, rama_prod, img_cut, image_text_areas):
        label_area = self.text_recognize_model.work_image_cut(
            image_text_areas, img_cut,
            self.prod_coefficients[rama_prod].number_coefficients.length,
            self.prod_coefficients[rama_prod].number_coefficients.height,
            self.prod_coefficients[rama_prod].number_coefficients.width,
            self.prod_coefficients[rama_prod].prod_coefficients.length,
            self.prod_coefficients[rama_prod].prod_coefficients.height,
            self.prod_coefficients[rama_prod].prod_coefficients.width,
            self.prod_coefficients[rama_prod].year_coefficients.length,
            self.prod_coefficients[rama_prod].year_coefficients.height,
            self.prod_coefficients[rama_prod].year_coefficients.width
        )
        res_labels = {}
        (number_text, number_conf) = label_area.labels[Class_text.number]
        if len(number_text) == self.prod_coefficients[rama_prod].number_coefficients.length:
            res_labels['number'] = (number_text, number_conf)
        else:
            number_image_text_areas = image_text_areas.areas[Class_text.number]
            image_text_areas_min = []
            image_text_areas_max = []
            for number_image_text_area in number_image_text_areas:
                w = number_image_text_area.xmax - number_image_text_area.xmin
                if (number_image_text_area.xmin - (w // 2)) < 0:
                    new_xmin = 0
                else:
                    new_xmin = number_image_text_area.xmin - (w // 2)
                rect_min = Rect(xmin=new_xmin, xmax=number_image_text_area.xmax, ymin=number_image_text_area.ymin,
                                ymax=number_image_text_area.ymax)
                image_text_areas_min.append(rect_min)
                new_xmax = number_image_text_area.xmax + w // 2
                rect_max = Rect(xmin=number_image_text_area.xmin, xmax=new_xmax, ymin=number_image_text_area.ymin,
                                ymax=number_image_text_area.ymax)
                image_text_areas_max.append(rect_max)
            image_text_areas.areas[Class_text.number] = image_text_areas_min
            label_area_min = self.text_recognize_model.work_image_cut(
                image_text_areas, img_cut,
                self.prod_coefficients[rama_prod].number_coefficients.length,
                self.prod_coefficients[rama_prod].number_coefficients.height,
                self.prod_coefficients[rama_prod].number_coefficients.width,
                self.prod_coefficients[rama_prod].prod_coefficients.length,
                self.prod_coefficients[rama_prod].prod_coefficients.height,
                self.prod_coefficients[rama_prod].prod_coefficients.width,
                self.prod_coefficients[rama_prod].year_coefficients.length,
                self.prod_coefficients[rama_prod].year_coefficients.height,
                self.prod_coefficients[rama_prod].year_coefficients.width
            )
            (number_text_min, number_conf_min) = label_area_min.labels[Class_text.number]
            if len(number_text_min) == self.prod_coefficients[rama_prod].number_coefficients.length:
                res_labels['number'] = (number_text_min, number_conf_min)
            else:
                image_text_areas.areas[Class_text.number] = image_text_areas_max
                label_area_max = self.text_recognize_model.work_image_cut(
                    image_text_areas, img_cut,
                    self.prod_coefficients[rama_prod].number_coefficients.length,
                    self.prod_coefficients[rama_prod].number_coefficients.height,
                    self.prod_coefficients[rama_prod].number_coefficients.width,
                    self.prod_coefficients[rama_prod].prod_coefficients.length,
                    self.prod_coefficients[rama_prod].prod_coefficients.height,
                    self.prod_coefficients[rama_prod].prod_coefficients.width,
                    self.prod_coefficients[rama_prod].year_coefficients.length,
                    self.prod_coefficients[rama_prod].year_coefficients.height,
                    self.prod_coefficients[rama_prod].year_coefficients.width
                )
                (number_text_max, number_conf_max) = label_area_max.labels[Class_text.number]
                if len(number_text_max) == self.prod_coefficients[rama_prod].number_coefficients.length:
                    res_labels['number'] = (number_text_max, number_conf_max)
                else:
                    res_labels['number'] = (number_text, number_conf)
        (year_text, year_conf) = label_area.labels[Class_text.year]
        if (len(year_text) == 2) and (int(year_text) < 25):
            res_labels['year'] = (year_text, year_conf)
        else:
            res_labels['year'] = ('23', 0.25)
        return res_labels





from danila.Danila_detail_CNN import Danila_detail_CNN


class Danila_detail():
    def __init__(self, danila_detail_params):
        self.danila_detail_params = danila_detail_params
        if (danila_detail_params.model_version == 1):
            self.danila_detail = Danila_detail_CNN(danila_detail_params.CNN_model_version)



    def detail_classify(self, img):
        return self.danila_detail.detail_classify(img)




import hashlib
import os
from datetime import datetime

import cv2

from data.neuro.Vagon_number_detect_class import Vagon_number_detect_class
from data.neuro.models import VAGON_NUMBER_DETECT_MODEL_ADDRESS, VAGON_NUMBER_DETECT_MODEL_ADDRESS_2


class Danila_vagon_text_detect_base:
    def __init__(self, yolov5_dir, vagon_text_detect_params):
        print('reading and loading - VAGON_NUMBER_DETECT_MODEL')
        if vagon_text_detect_params.vagon_text_detect_version == 1:
            self.vagon_number_detect_model = Vagon_number_detect_class(VAGON_NUMBER_DETECT_MODEL_ADDRESS, yolov5_dir)
        elif vagon_text_detect_params.vagon_text_detect_version == 2:
            self.vagon_number_detect_model = Vagon_number_detect_class(VAGON_NUMBER_DETECT_MODEL_ADDRESS_2, yolov5_dir)

    def vagon_number_detect(self, img):
        hash_object = hashlib.md5(bytes(str(datetime.now()), 'utf-8'))
        hash_str = hash_object.hexdigest()
        initial_image_path = 'initial_image' + hash_str + '.jpg'
        cv2.imwrite(initial_image_path, img)
        number_rects = self.vagon_number_detect_model.vagon_rama_detect(initial_image_path, 512)
        if len(number_rects) == 0:
            os.remove(initial_image_path)
            return img
        img_with_number = img.copy()
        for number_rect in number_rects:
            cv2.rectangle(img_with_number, (number_rect.xmin, number_rect.ymin), (number_rect.xmax, number_rect.ymax), (0, 0, 255), 2)
            cv2.putText(img_with_number, 'number', (number_rect.xmin, number_rect.ymin), cv2.FONT_HERSHEY_SIMPLEX, 0.5, (0, 0, 255), 2)
        os.remove(initial_image_path)
        return img_with_number

    def vagon_number_detect_rects(self, img):
        hash_object = hashlib.md5(bytes(str(datetime.now()), 'utf-8'))
        hash_str = hash_object.hexdigest()
        initial_image_path = 'initial_image' + hash_str + '.jpg'
        cv2.imwrite(initial_image_path, img)
        number_rects = self.vagon_number_detect_model.vagon_rama_detect(initial_image_path, 512)
        os.remove(initial_image_path)
        return number_rects


from data.result import Rama_prod, balka_prod

RAMA_PRODS = {Rama_prod.Rama_Prod.altai: '1923',
              Rama_prod.Rama_Prod.balakovo:'1487',
              Rama_prod.Rama_Prod.begickaya:'12',
              Rama_prod.Rama_Prod.promlit:'33',
              Rama_prod.Rama_Prod.ruzhimmash:'1275',
              Rama_prod.Rama_Prod.tihvin:'1378',
              Rama_prod.Rama_Prod.uralvagon:'5'}

BALKA_PRODS = {balka_prod.Balka_Prod.altai: '1923',
              balka_prod.Balka_Prod.begickaya:'12',
              balka_prod.Balka_Prod.promlit:'33',
              balka_prod.Balka_Prod.ruzhimmash:'1275',
              balka_prod.Balka_Prod.tihvin:'1378'}


from data.neuro.Rama_prod_classify_class import Rama_prod_classify_class
from data.neuro.models import RAMA_PROD_CLASSIFY_MODEL_ADDRESS, RAMA_PROD_CLASSIFY_MODEL_ADDRESS_2


class Danila_rama_classify_prod:
    def __init__(self, yolov5_dir, rama_classify_model):
        yolo_path = yolov5_dir
        if rama_classify_model == 1:
            rama_prod_classify_model_path = RAMA_PROD_CLASSIFY_MODEL_ADDRESS
        else:
            rama_prod_classify_model_path = RAMA_PROD_CLASSIFY_MODEL_ADDRESS_2
        print('reading and loading - RAMA_PROD_CLASSIFY_MODEL')
        self.rama_prod_classify_model = Rama_prod_classify_class(rama_prod_classify_model_path, yolo_path)

    def rama_classify(self, img):
        rama_prod_conf = self.rama_prod_classify_model.classify(img)
        return rama_prod_conf